/* $Id: analyze.c,v 2.0 94/07/17 18:33:40 dfk Exp $ */

#include "modularize.h"
#include "queue.h"

#include "diskdevice.h"
#define LEN 256
#define BQSIZE 15
#define WRITE 1
#define READ 0
#define MILL 1000000		/* 10^6 */
#define BQSIZE 15

/* ----------------------- PRIVATE Prototypes --------------------- */
PRIVATE TICS LongToTICS(long sec,long start);

/* -------------------------- Source -----------------------------  */


Heap heap;

int
main(argc,argv)
               int argc;
               char *argv[];
{
  FILE *inputF,*OutputDataF;
  char *outputFname = argv[1];
  TICS absstart = 0;
  TICS absstop = 0;
  TICS real_time = 0; /* all in usec i.e. 10^6 secs */
  TICS last_absstop = 0;
  TICS wait_time = 0;
  NCQUEUE *busWaitq;
  char busString[] = "bus queue";
  int busOwner;
  ulong sectorno = 0;
  ulong sec = 0;
  ulong start = 0;
  ulong stop = 0;
  ulong size = 0;
  int write_op  = 0;
  int sync  = 0;
  int metadata  = 0;
  int done = 0;
  int DISK = 0;

if (argc != 2)
   {
   fprintf(stderr,"usage: %s OutputFileName \n",argv[0]);
exit(1);
}
                                 /* Open the output file */
  if ((OutputDataF = fopen(argv[1],"w")) ==  NULL)
    printf("Error in opening file :  %s\n",outputFname);
  
 
  inputF = stdin;		/* input from file piped in at prompt */

  /* printf("Check 1: All ok \n"); */
  
  InitTime();

  busOwner = BUS_FREE;
  busWaitq = MakeQueue_noncyc(BQSIZE,busString);

  DiskDeviceInit(DISK,DISK,&busOwner,busWaitq, NULL);

  /* printf("Check 2: All ok \n"); */
                                /* read in info from perl's sample output */
for (done = 0;done != -1;)
    {
      TICS model_start= 0;
      TICS model_stop = 0;
      TICS model_time = 0;


      done =  fscanf(inputF,"%lu %lu %lu %lu %lu %d %d %d",
		     &sectorno,&sec,&start,&stop,&size,
		     &write_op,&metadata,&sync);
      if (done != -1)
	{
	  char *dummy = NULL;
#ifdef SRI_DEBUG
	  CheckPt1(sectorno,sec,start,stop,size,write_op); 
#endif SRI_DEBUG
	                            /* convert to TICS time*/
	  absstart = LongToTICS(sec,start);
	  absstop = LongToTICS(sec,stop);	  
	  real_time = (absstop - absstart);
	  wait_time = (absstart - last_absstop);
	  printf(" absstart = %s; last_absstop = %s \n",time_print(absstart),
	  time_print(last_absstop));

	  if (absstart < last_absstop) {
	      fflush(stdout);
	      fprintf(stderr, "WHOOPS wait time is negative\n");
	      wait_time = 0;
	  }
	  last_absstop = absstop;

#ifdef SRI_DEBUG
			/* just to make sure this works */
	  printf("absstart = %s: absstop = %s:  real time (TICS) = %s\n",
		 time_print(absstart), time_print(absstop),
		 time_print(real_time));
	  /*
	     printf("simple time difference in usec = %s\n",
	             time_print(stop-start));
	     printf("simple time difference in TICS = %s\n",
	             time_print((absstop-absstart)) );
	  */
#endif
	  
	  WaitDuration((TICS) wait_time); 
/* 	  WaitTime((TICS) absstart); */

/*	  printf("wait_time = %s\n", time_print(wait_time)); */
 	          /* absstart and model_start shd be the same! */
	  model_start = GetTime();

	  DiskDeviceTransfer(DISK,sectorno,size/512,write_op,dummy);
	  if ((metadata && sync) && write_op)
	    DiskDeviceSync(DISK);

	  model_stop = GetTime();
	  model_time = model_stop - model_start;

#ifdef SRI_DEBUG
	  printf("model start = %s, model stop = %s, model time = %s\n",
		 time_print(model_start), time_print(model_stop), 
		 time_print(model_time));
#endif
	  if (model_time > (150. * TICSperMSEC)) {
	      fflush(stdout);
	      fprintf(stderr, "******** model time is huge ***\n");
	  }

/* OutputData Format
Real_time      Model_time      Sectorno       No.of bytes       Read/Write
*/
	  fprintf(OutputDataF,"%s     \t %s     \t %lu     \t %lu     \t %d\n",
		  time_print(real_time),time_print(model_time),
		  sectorno,size,write_op);
	

	} /* if not done */
      
    }/* while not done */
  
  DiskDeviceSync(DISK);
  DiskDeviceDone(DISK);

  fclose(inputF);
  fclose(OutputDataF);

  exit(0);
} /* main */



PRIVATE TICS LongToTICS(long sec,long start) 
{
TICS temp = 0;
TICS LLsec = sec;
TICS LLstart = start;

temp =  (TICS)( ((LLsec * MILL) + LLstart) * (TICSperMSEC/1000));
return temp;
}
