/* 
 *  @TITLE "test2b.c"
 *
 *  This file contains function that reads from a text file and
 *  put the data into an intermediate buffer, then it writes the
 *  intermediate buffer to the virtual disk by calling the functions
 *  in the disk module.
 *
 *  It also contains a function that reads from the abovementioned
 *  virtual disk through the interface functions into the intermediate
 *  buffer and feeds the data to an output file.
 *
 *         Song Bac Toh 
 *         July 7, 1994
 */
/* $Id: test2b.c,v 2.0 94/07/17 18:35:26 dfk Exp $ */


#include "modularize.h"
#include "diskdevice.h"

#define BUFFERSIZE        512	/* size of intermediate buffer */
#define SECTORperBUFFER  ( BUFFERSIZE / 512 ) /* sector size is 512 */
char inBuffer[BUFFERSIZE];
static int count;
#define OFFSET            256	/* skipping half a block */

int copy2VDisk(char *fileName, int diskID);
int copyfromVDisk(int diskID, char *fileName);

/* ====================================================================== */
/* copy2VDisk
 *
 * Read text from file to buffer, then write the buffer into the 
 * specified vdisk.  
 *     Error => return -1.
 *     OK    => return 0.
 *     Abnormal => positive number.
 */
int copy2VDisk(char *fileName, int diskID)
{
  FILE *fp;
  int ret, index;
  extern int count;
  long fileSize;

  if ((fp = fopen(fileName, "r")) == NULL)
    {
      perror("copy2VDisk: cannot open requested file");
      exit(FATAL);
    }
  else				/* get the file size */
    {
      fseek(fp, 0L, SEEK_END);	/* go to the end of file */
      fileSize = ftell(fp);	/* get the file size */
      rewind(fp);
    }

  count = 0;
  for (index = 0; index < BUFFERSIZE; index++)
    inBuffer[index] = '~';
  inBuffer[0] = '~';

  /* assuming we never encounter error so if return value is zero, then
     we have EOF */
  while (1)
    {
      if (fileSize < BUFFERSIZE)
	ret = fread(inBuffer, sizeof(char), fileSize, fp);
      else
	ret = fread(inBuffer, sizeof(char), BUFFERSIZE, fp);
      if (ret == 0)		/* we have reached EOF */
	break;
      

      /* DiskDeviceTransfer(diskID, (count * SECTORperBUFFER), SECTORperBUFFER,
			 1, inBuffer); */
      DiskDeviceStart(diskID, (count * SECTORperBUFFER), SECTORperBUFFER, 1);
      DiskDeviceFinish(diskID, inBuffer);
      count++;
      fileSize -= ret;		/* fileSize = how much is remaining */

      for (index = 0; index < BUFFERSIZE; index++)
	inBuffer[index] = '~';
    }
  return(ret);
}



/* copyfromVDisk
 *
 * Read text from vdisk to buffer, then write the buffer into the 
 * specified file.  
 *     Error => return -1.
 *     OK    => return 0.
 *     Abnormal => positive number.
 */
int copyfromVDisk(int diskID, char *fileName)
{
  FILE *fp;
  int index = 0;

  if ((fp = fopen(fileName, "w")) == NULL)
    {
      perror("copyfromVDisk:  cannot open requested file");
      exit(FATAL);
    }
  rewind(fp);

  /* assuming we never encounter error so if return value is zero, then
     we have EOF */
  while (1)
    {
      DiskDeviceTransfer(diskID, (index * SECTORperBUFFER), SECTORperBUFFER,
			 0, inBuffer); 
      /*
     DiskDeviceStart(diskID, (index * SECTORperBUFFER), SECTORperBUFFER, 0);
     DiskDeviceFinish(diskID, inBuffer);
      */
      index++;  count--;
      
      if (count < 0) 
	return(0);
      fwrite(inBuffer, sizeof(char), BUFFERSIZE, fp);
    }
}
